@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use 5.010; # //
 
-our $VERSION = '0.63';
+our $VERSION = '0.64';
 
 use Carp;
 
@@ -372,26 +372,6 @@ than twice as would be the case calling the above methods individually.
 
 =cut
 
-sub set_on_geom_changed
-{
-   croak "\$win->set_on_geom_changed is deprecated; use ->bind_event";
-}
-
-sub set_on_key
-{
-   croak "\$win->set_on_key is deprecated; use ->bind_event";
-}
-
-sub set_on_mouse
-{
-   croak "\$win->set_on_mouse is deprecated; use ->bind_event";
-}
-
-sub set_on_expose
-{
-   croak "\$win->set_on_expose is deprecated; use ->bind_event";
-}
-
 our $INDENT = "";
 sub _do_expose
 {
@@ -446,11 +426,6 @@ window.
 
 =cut
 
-sub set_on_focus
-{
-   croak "\$win->set_on_focus is deprecated; use ->bind_event";
-}
-
 =head2 set_focus_child_notify
 
    $win->set_focus_child_notify( $notify )
@@ -834,16 +809,6 @@ Controls whether this window is currently stealing input from its siblings
 
 =cut
 
-sub clearline
-{
-   croak "\$win->clearline has been removed";
-}
-
-sub clear
-{
-   croak "\$win->clear has been removed";
-}
-
 sub sprintf
 {
    my $self = shift;
